#ifndef TT_SQUIRREL_CLASS_BINDER_H_
#define TT_SQUIRREL_CLASS_BINDER_H_

#include "squirrel.h"

#include "tt_squirrel_object.h"
#include "tt_squirrel_utility.h"
#include "tt_squirrel_virtual_machine.h"


namespace TtSquirrel {
  template <class TYPE>
  class ClassBinder {
  public:
    typedef int (*FunctionBinder)( VirtualMachine& vm, TYPE& object );

    static const int ExistReturnValue = 1;
    static const int NoReturnValue = 0;

    class ArgumentsCountError {};

  private:
    static int NativeClosure( HSQUIRRELVM vm ) {
      FunctionBinder function_binder = reinterpret_cast<FunctionBinder>(
        NativeAPI( vm ).GetAsUserPointer( TtSquirrel::StackTop ) );
      NativeAPI( vm ).PopTop();
      TYPE* object = reinterpret_cast<TYPE*>(
        NativeAPI( vm ).GetInstanceUp( TtSquirrel::StackBase, TypeTag( typeid( TYPE ) ) ) );
      try {
        return (*function_binder)( VirtualMachine( vm ), *object );
      }
      catch ( ArgumentsCountError ) {
        return NativeAPI( vm ).ThrowError( "wrong number of arguments" );
      }
    }

  protected:
    static void CheckArgumentsCount( VirtualMachine& vm, unsigned int count ) {
      if ( vm.Native().GetTop() != static_cast<int>( count + 1 ) ) {
        throw ArgumentsCountError();
      }
    }

  public:
    virtual ~ClassBinder() {}

    explicit ClassBinder( VirtualMachine& vm ) :
    vm_( vm ),
    object_of_class_( (vm.Native().NewClass( false ),
                       vm.GetStackObject( TtSquirrel::StackTop )) ),
    type_tag_( typeid( TYPE ) ) {
      vm_.Native().SetTypeTag( TtSquirrel::StackTop, type_tag_ );
      vm_.Native().PopTop();
    }

    Object::Class& GetObjectOfClass( void ) {
      return object_of_class_;
    }

    void MakeInstance( TYPE& object ) {
      vm_.PushObject( object_of_class_ );
      vm_.PushObject( object_of_class_ );
      vm_.Native().Call( 1 );
      vm_.Native().SetInstanceUp( TtSquirrel::StackTop, &object );
      vm_.Native().Remove( TtSquirrel::StackTop - 1 );
    }

    void RegistMethodByName( const std::string& name, FunctionBinder function_binder ) {
      vm_.Native().PushString( name );
      this->RegistMethod( function_binder );
    }

    void RegistMethod( FunctionBinder function_binder ) {
      vm_.PushObject( object_of_class_ );
      vm_.Native().Push( Pushed( TtSquirrel::StackTop, 1 ) );
      vm_.Native().PushUserPointer( function_binder );
      vm_.Native().NewClosure( NativeClosure, 1 );
      vm_.Native().NewSlot( Pushed( TtSquirrel::StackTop, 2 ),false );
      vm_.Native().Pop( 2 );
    }

  private:
    VirtualMachine& vm_;
    Object::Class   object_of_class_;
    TypeTag         type_tag_;
  };
}

#endif // TT_SQUIRREL_CLASS_BINDER_H_
