#include "tt_squirrel_exception.h"

using namespace TtSquirrel;


// -- Exception ----------------------------------------------------------

// -- NativeAPIException -------------------------------------------------
NativeAPIException::NativeAPIException( const std::string& function_name, const std::string& reason ) :
function_name_( function_name ),
reason_( reason )
{
}

const std::string&
NativeAPIException::GetFunctionName( void )
{
  return function_name_;
}

const std::string&
NativeAPIException::GetReason( void )
{
  return reason_;
}

// -- ObjectTypeException ------------------------------------------------
ObjectTypeException::ObjectTypeException( ObjectType actual, ObjectType require ) :
actual_( actual ),
require_( require )
{
}

ObjectType
ObjectTypeException::GetActualType( void ) const
{
  return actual_;
}

ObjectType
ObjectTypeException::GetRequireType( void ) const
{
  return require_;
}


// -- ObjectHasNotSlotException ------------------------------------------
ObjectHasNotSlotException::ObjectHasNotSlotException( ObjectType actual ) :
actual_( actual )
{
}

ObjectType
ObjectHasNotSlotException::GetActualType( void ) const
{
  return actual_;
}


// -- GetOperationException ----------------------------------------------
GetOperationException::GetOperationException( const std::string& key_info ) :
WithString( key_info )
{
}

const std::string&
GetOperationException::GetKeyInfo( void ) const
{
  return this->GetString();
}

// -- SetOperationException ----------------------------------------------
SetOperationException::SetOperationException( const std::string& key_info ) :
WithString( key_info )
{
}

const std::string&
SetOperationException::GetKeyInfo( void ) const
{
  return this->GetString();
}

// -- FileExecutionException -------------------------------------------
FileExecutionException::FileExecutionException( const std::string& filename ) :
WithString( filename )
{
}


const std::string&
FileExecutionException::GetFileName( void ) const
{
  return this->GetString();
}
