#ifndef TT_SQUIRREL_EXCEPTION_H_
#define TT_SQUIRREL_EXCEPTION_H_

#include <string>

// #include "tt_exception.h"

#include "tt_squirrel_object_type.h"


namespace TtSquirrel {

  // dpp
  class WithString {
  public:
    explicit WithString( const std::string& str ) : str_( str ) {}
    const std::string& GetString( void ) const { return str_; }
  private:
    std::string str_;
  };

  // -- Exception --------------------------------------------------------
  // class Exception : public TtException {};
  class Exception {};

  // -- NativeAPIException -----------------------------------------------
  class NativeAPIException : public Exception {
  public:
    explicit NativeAPIException( const std::string& function_name, const std::string& reason );

    const std::string& GetFunctionName( void );
    const std::string& GetReason( void );

  private:
    std::string function_name_;
    std::string reason_;
  };

  // -- ObjectTypeException ----------------------------------------------
  class ObjectTypeException : public Exception {
  public:
    explicit ObjectTypeException( ObjectType actual, ObjectType require );

    ObjectType GetActualType( void ) const;
    ObjectType GetRequireType( void ) const;

  private:
    ObjectType actual_;
    ObjectType require_;
  };

  // -- ObjectHasNotSlotException ----------------------------------------
  class ObjectHasNotSlotException : public Exception {
  public:
    explicit ObjectHasNotSlotException( ObjectType actual );

    ObjectType GetActualType( void ) const;

  private:
    ObjectType actual_;
  };

  // -- GetOperationException --------------------------------------------
  class GetOperationException : public Exception, private WithString {
  public:
    explicit GetOperationException( const std::string& key_info );

    const std::string& GetKeyInfo( void ) const;
  };

  // -- SetOperationException --------------------------------------------
  class SetOperationException : public Exception, private WithString {
  public:
    explicit SetOperationException( const std::string& key_info );

    const std::string& GetKeyInfo( void ) const;
  };

  // -- FileExecutionException -------------------------------------------
  class FileExecutionException : public Exception, private WithString {
  public:
    explicit FileExecutionException( const std::string& filename );

    const std::string& GetFileName( void ) const;
  };
}

#endif // TT_SQUIRREL_EXCEPTION_H_
