#include "ttl_define.h"

#include "tt_squirrel_exception.h"
#include "tt_squirrel_native_api.h"

#include "tt_squirrel_object.h"

#include "tt_squirrel_virtual_machine.h"

using namespace TtSquirrel;


// -- Object -------------------------------------------------------------
Object::Object( VirtualMachine& vm, int index ) :
object_(),
vm_( vm ),
need_release_( true )
{
  vm_.Native().GetStackObject( index, &object_ );
  vm_.Native().AddReference( &object_ );
}


Object::Object( const Object& other ) :
object_( other.object_ ),
vm_( other.vm_ ),
need_release_( other.need_release_ )
{
  if ( need_release_ ) {
    vm_.Native().AddReference( &object_ );
  }
}


Object&
Object::operator =( const Object& other )
{
  if ( need_release_ ) {
    this->Release();
  }
  object_ = other.object_;
  vm_ = other.vm_;
  need_release_ = other.need_release_;
  if ( need_release_ ) {
    vm_.Native().AddReference( &object_ );
  }
  return *this;
}


Object::~Object()
{
  if ( need_release_ ) {
    this->Release();
  }
}


HSQOBJECT
Object::GetHandle( void )
{
  return object_;
}

VirtualMachine&
Object::GetVirtualMachine( void )
{
  return vm_;
}

ObjectType
Object::GetType( void ) const
{
  return object_._type;
}

void
Object::Release( void )
{
  vm_.Native().Release( &object_ );
}


// -- Object::Null -------------------------------------------------------
// -- Object::Integer ----------------------------------------------------
// -- Object::Float ------------------------------------------------------
// -- Object::Boolean ----------------------------------------------------
// -- Object::String -----------------------------------------------------
// -- Object::Table ------------------------------------------------------
// -- Object::Array ------------------------------------------------------
// -- Object::UserData ---------------------------------------------------
// -- Object::Closure ----------------------------------------------------
// -- Object::NativeClosure ----------------------------------------------
// -- Object::Generator --------------------------------------------------
// -- Object::UserPointer ------------------------------------------------
// -- Object::Thread -----------------------------------------------------
// -- Object::FuncProto --------------------------------------------------

// -- Object::Class ------------------------------------------------------
Object::Class::Class( const Object& other ) :
Object( other )
{
  if ( other.GetType() != ObjectType::Class ) {
    throw ObjectTypeException( other.GetType(), ObjectType::Class );
  }
}

// -- Object::Instance ---------------------------------------------------
// -- Object::WeakReferrence ---------------------------------------------
