#ifndef TT_SQUIRREL_UTILITY_H_
#define TT_SQUIRREL_UTILITY_H_

#include <map>

#include "squirrel.h"

#include "tt_squirrel_object_type.h"


namespace TtSquirrel {
  const int StackTop  = -1;
  const int StackBase = 1;

  int Pushed( int index, int pushed );

  void DefaultPrintFunc( HSQUIRRELVM v, const SQChar* s, ... );

  void ObjectTypeCheck( ObjectType require, ObjectType actual );


  // -- TypeTag ----------------------------------------------------------
  class TypeTag {
  private:
    static std::map<std::string, void*> TABLE;
  public:
    TypeTag( const std::type_info& type );
    void* GetTag( void ) const;
    operator void*( void );
  private:
    void* tag_;
  };


  // -- Operation --------------------------------------------------------
  
  
  template <class OBJECT, class FUNCTION>
  class Operation0 {
  public:
    explicit Operation0( OBJECT object, FUNCTION function ) :
    object_( object ), function_( function ) {}
    void operator ()( void ) {
      (object_.*function)();
    }
  private:
    OBJECT&  object_;
    FUNCTION function_;
  };

  template <class OBJECT, class FUNCTION, class ARG1>
  class Operation1 {
  public:
    explicit Operation1( OBJECT object, FUNCTION function, ARG1 arg1 ) :
    object_( object ), function_( function ) {}
    void operator ()( void ) {
      (object_.*function)( arg1_ );
    }
  private:
    OBJECT&  object_;
    FUNCTION function_;
    ARG1     arg1_;
  };
}

#endif // TT_SQUIRREL_UTILITY_H_
