#ifndef TT_SQUIRREL_VIRTUAL_MACHINE_H_
#define TT_SQUIRREL_VIRTUAL_MACHINE_H_

#include "squirrel.h"

#include "tt_squirrel_native_api.h"
#include "tt_squirrel_object.h"
#include "tt_squirrel_operation.h"

#include "ttl_define.h"

namespace TtSquirrel {
  // -- VirtualMachine ---------------------------------------------------
  class VirtualMachine {
  public:
    explicit VirtualMachine( int stack_size = 1024 );
    explicit VirtualMachine( HSQUIRRELVM vm, bool need_release_ = false );
    virtual ~VirtualMachine();

    HSQUIRRELVM GetHandle( void );

    NativeAPI Native( void );

    // -- bundle ---------------------------------------------------------
    void GetByString( int index, const std::string& key );
    void GetByInteger( int index, int key );

    void PushObject( Object& object );


    // -- complete set ---------------------------------------------------
    void NewSlot( const Operation& table, const Operation& key, const Operation& value );

    void Set( const Operation& table, const Operation& key, const Operation& value );
    void Get( const Operation& table, const Operation& key );
    void GetByString( const Operation& table, const std::string& key );
    void GetByInteger( const Operation& table, int key );

    void DoFile( const std::string& filename, bool print_error = true );

    Object GetStackObject( int index );

  private:
    HSQUIRRELVM vm_;
    bool        need_release_;
  };
}

#endif // TT_SQUIRREL_VIRTUAL_MACHINE_H_
