#ifndef TT_COMBO_BOX_H_
#define TT_COMBO_BOX_H_

#include "tt_window.h"

// -- TtComboBox ---------------------------------------------------------
class TtComboBox : public BasedOnTtWindow<WS_TABSTOP, WS_EX_CLIENTEDGE, TtWindowClassName::COMBOBOX> {
public:
  int GetCount( void );
  std::string GetTextAt( unsigned int index );
  int Find( unsigned int start, const std::string& str );

  int Push( const std::string& str );
  int InsertAt( unsigned int index, const std::string& str );
  int DeleteAt( unsigned int index );
  void Clear( void );

  int GetSelectedIndex( void );
  std::string GetSelectedString( void );
  int SetSelect( unsigned int index );
  int SetSelectByString( const std::string& str );
};

// -- TtComboBoxDropDownWithEdit -----------------------------------------
typedef TtWindowWithStyle<TtComboBox, CBS_DROPDOWN>
TtComboBoxDropDownWithEdit;

// -- TtComboBoxDropDownNotEdit ------------------------------------------
typedef TtWindowWithStyle<TtComboBox, CBS_DROPDOWNLIST>
TtComboBoxDropDownNotEdit;

// -- TtComboBoxTable ----------------------------------------------------
template <class ComboBox, class Value>
class TtComboBoxTable : public ComboBox {
public:
  Value GetItemData( unsigned int index ) {
    return static_cast<Value>( this->SendMessage( CB_GETITEMDATA, index ) );
  }

  Value GetSelectedItemData( void ) {
    return static_cast<Value>( this->GetItemData( this->GetSelectedIndex() ) );
  };

  void SetItemData( unsigned int index, Value data ) {
    this->SendMessage( CB_SETITEMDATA, index, static_cast<LPARAM>( data ) );
    table_[data] = this->GetTextAt( index );
    this->SetAlias( data, data );
  }

  void PushWithData( const std::string& str, Value data ) {
    this->Push( str );
    this->SetItemData( this->GetCount() - 1, data );
    table_[data] = str;
  }

  void SetSelectByItemData( Value data ) {
    this->SetSelectByString( table_[alias_table_[data]] );
  }

  void SetAlias( Value other, Value real ) {
    alias_table_[other] = real;
  }

private:
  std::map<Value, std::string> table_;
  std::map<Value, Value>       alias_table_;
};

// -- TtComboBoxTableDropDownWithEdit ------------------------------------
template <class V>
class TtComboBoxTableDropDownWithEdit : public TtComboBoxTable<TtComboBoxDropDownWithEdit, V> {
};

// -- TtComboBoxTableDropDownNotEdit -------------------------------------
template <class V>
class TtComboBoxTableDropDownNotEdit : public TtComboBoxTable<TtComboBoxDropDownNotEdit, V> {
};

#endif // TT_COMBO_BOX_H_
