#ifndef TT_DYNAMIC_LINK_LIBRARY_H_
#define TT_DYNAMIC_LINK_LIBRARY_H_

#include <windows.h>

#include <vector>

#include "tt_exception.h"

class TtDynamicLinkLibrary {
public:
  explicit TtDynamicLinkLibrary( const std::string& filename );
  ~TtDynamicLinkLibrary();

  const std::vector<std::string>& GetFunctionNameList( void );

  template <class FUNCTION_TYPE>
  FUNCTION_TYPE GetFunction( const std::string& name ) {
    return reinterpret_cast<FUNCTION_TYPE>( this->GetProcessAddress( name.c_str() ) );
  }

  template <class FUNCTION_TYPE>
  FUNCTION_TYPE GetFunction( unsigned int index ) {
    if ( index >= function_name_list_.size() ) {
      throw TtInvalidOperationException( typeid( *this ) );
    }
    return reinterpret_cast<FUNCTION_TYPE>( this->GetProcessAddress( function_name_list_[index].c_str() ) );
  }

private:
  FARPROC GetProcessAddress( const char* name );

private:
  HINSTANCE                handle_;
  LPBYTE                   base_;
  std::vector<std::string> function_name_list_;
};

#endif // TT_DYNAMIC_LINK_LIBRARY_H_
