#ifndef TT_INI_FILE_H_
#define TT_INI_FILE_H_

#include <string>

#include "tt_shared_pointer.h"

class TtIniFile;
class TtIniSection;

// -- TtIniFile ----------------------------------------------------------
class TtIniFile {
public:
  static const int DEFAULT_BUFFER_SIZE = 65536;

  // ftHg exe Ɠt@C .ini ̃t@C
  explicit TtIniFile( void );
  explicit TtIniFile( const std::string& filename );
  explicit TtIniFile( unsigned int buffer_size );
  explicit TtIniFile( const std::string& filename, unsigned int buffer_size );

  ~TtIniFile();

  const std::string& GetFileName( void ) const;
  void SetFileName( const std::string& filename );

  TtIniSection GetSection( const std::string& name );
  TtIniSection operator []( const std::string& name );

  void Flush( void );

private:
  std::string    filename_;
  TtSharedString buffer_;
};


// -- TtIniSection -------------------------------------------------------
class TtIniSection {
  friend class TtIniFile;

private:
  explicit TtIniSection( TtIniFile& ini_file, const std::string& name, TtSharedString& buffer );

public:
  explicit TtIniSection( const TtIniSection& obj );

  TtIniFile& GetIniFile( void ) const;

  const std::string& GetName( void ) const;

  bool GetBoolean( const std::string& key );
  bool GetBoolean( const std::string& key, bool default_value );
  void SetBoolean( const std::string& key, bool value );

  int GetInteger( const std::string& key );
  int GetInteger( const std::string& key, int default_value );
  void SetInteger( const std::string& key, int value );

  double GetDouble( const std::string& key );
  double GetDouble( const std::string& key, double default_value );
  void SetDouble( const std::string& key, double value );

  std::string GetString( const std::string& key );
  std::string GetString( const std::string& key, const std::string& default_value );
  void SetString( const std::string& key, const std::string& value );

  void Flush( void );

private:
  TtIniFile&      ini_file_;
  std::string     name_;
  TtSharedString& buffer_;
};

#endif // TT_INI_FILE_H_
