#ifndef TT_LISTVIEW_H_
#define TT_LISTVIEW_H_

#include "tt_window.h"

class TtListView;

// -- TtListViewColumn ---------------------------------------------------
class TtListViewColumn {
  friend class TtListView;

private:
  explicit TtListViewColumn( TtListView& list_view, int index );

  void GetInfo( LVCOLUMN& info ) const;
  void SetInfo( LVCOLUMN& info );

public:
  int GetTextAlign( void ) const;
  void SetTextAlign( int format );

  int GetIndex( void ) const;

  int GetWidth( void ) const;
  void SetWidth( int width );

  std::string GetText( void ) const;
  void SetText( const std::string& text );

  void Remove( void );

private:
  TtListView& list_view_;
  int         index_;
};

// -- TtListViewItem -----------------------------------------------------
class TtListViewItem {
  friend class TtListView;

private:
  explicit TtListViewItem( TtListView& list_view, int index );

  void GetInfo( LVITEM& info ) const;
  void SetInfo( LVITEM& info );

public:
  TtListViewItem operator =( const TtListViewItem& obj );

  bool GetChecked( void ) const;
  void SetChecked( bool flag );

  bool GetFocused( void ) const;
  void SetFocused( bool flag );

  bool GetSelected( void ) const;
  void SetSelected( bool flag );
  void SetSelectionMark( void );

  std::string GetText( void ) const;
  void SetText( const std::string& text );

  std::string GetSubItemText( int index ) const;
  void SetSubItemText( int index, const std::string& text );

  int GetIndex( void ) const;

  int GetImageIndex( void ) const;
  void SetImageIndex( int index );

  void BeginEdit( void );
  void Remove( void );

  template <class TYPE>
  TYPE GetData( void ) const {
    LVITEM tmp = {LVIF_PARAM};
    tmp.iItem = index_;
    this->GetInfo( tmp );
    return reinterpret_cast<TYPE>( tmp.lParam );
  }
  template <class TYPE>
  void SetData( TYPE data ) {
    LVITEM tmp = {LVIF_PARAM};
    tmp.iItem = index_;
    tmp.lParam = data;
    this->SetInfo( tmp );
  }

private:
  TtListView& list_view_;
  int         index_;
};


// -- TtListView ---------------------------------------------------------
class TtListView : public BasedOnTtWindow<WS_TABSTOP | LVS_SHAREIMAGELISTS,  WS_EX_CLIENTEDGE, TtWindowClassName::LISTVIEW> {
public:
  explicit TtListView( void );

  int GetView( void );
  void SetView( int view );

  bool GetHasCheckBox( void );
  void SetHasCheckBox( bool flag );

  bool GetHasGridLines( void );
  void SetHasGridLines( bool flag );

  bool GetLabelEdit( void );
  void SetLabelEdit( bool flag );

  bool GetScrollable( void );
  void SetScrollable( bool flag );

  bool GetMultiselect( void );
  void SetMultiselect( bool flag );

  bool GetFullRowSelect( void );
  void SetFullRowSelect( bool flag );

  bool GetHideSelection( void );
  void SetHideSelection( bool flag );

  int GetSorting( void );
  void SetSorting( int sorting );

  int GetHeaderStyle( void );
  void SetHeaderStyle( int style );

  TtImageList GetSmallImageList( void );
  void SetSmallImageList( TtImageList& image_list );

  TtImageList GetLargeImageList( void );
  void SetLargeImageList( TtImageList& image_list );

  int GetColumnCount( void );
  TtListViewColumn MakeNewColumn( void );
  TtListViewColumn GetColumn( int index );

  int GetItemCount( void );
  TtListViewItem MakeNewItem( void );
  TtListViewItem GetItem( int index );

  std::vector<int> GetCheckedIndices( void );
  std::vector<TtListViewItem> GetCheckedItems( void );

  std::vector<int> GetFocusedIndices( void );
  std::vector<TtListViewItem> GetFocusedItems( void );

  int GetSelectedCount( void );
  std::vector<int> GetSelectedIndices( void );
  std::vector<TtListViewItem> GetSelectedItems( void );

  void Arrange( int format );

  void LockDraw( void );
  void UnlockDraw( void );

  void EnsureVisible( int index );

  // ۑ
  // void Sort( void );

  void Clear( void );

private:
  int column_count_;
};

typedef TtWindowWithStyle<TtListView, LVS_ICON>      TtListViewIcon;
typedef TtWindowWithStyle<TtListView, LVS_SMALLICON> TtListViewSmallIcon;
typedef TtWindowWithStyle<TtListView, LVS_LIST>      TtListViewList;
typedef TtWindowWithStyle<TtListView, LVS_REPORT>    TtListViewReport;

#endif // TT_LISTVIEW_H_
