#ifndef TT_MESSAGE_BOX_H_
#define TT_MESSAGE_BOX_H_

#include <string>
#include <map>

#include <windows.h>

#include "tt_window.h"
#include "tt_string.h"

#ifdef ERROR
#undef ERROR
#endif

#ifdef IGNORE
#undef IGNORE
#endif

// -- TtMessageBox -------------------------------------------------------
class TtMessageBox {
public:
  class Icon {
    static const unsigned int NONE        = 0;
    static const unsigned int EXCLAMATION = MB_ICONEXCLAMATION;
    static const unsigned int WARNING     = MB_ICONWARNING;
    static const unsigned int INFORMATION = MB_ICONINFORMATION;
    static const unsigned int ASTERISK    = MB_ICONASTERISK;
    static const unsigned int QUESTION    = MB_ICONQUESTION;
    static const unsigned int STOP        = MB_ICONSTOP;
    static const unsigned int ERROR       = MB_ICONERROR;
    static const unsigned int HAND        = MB_ICONHAND;
  };

  class DefautButton {
    static const unsigned int FIRST  = MB_DEFBUTTON1;
    static const unsigned int SECOND = MB_DEFBUTTON2;
    static const unsigned int THIRD  = MB_DEFBUTTON3;
    static const unsigned int FOURTH = MB_DEFBUTTON4;
  };

  class Result {
    static const unsigned int ABORT  = IDABORT;
    static const unsigned int CANCEL = IDCANCEL;
    static const unsigned int IGNORE = IDIGNORE;
    static const unsigned int NO     = IDNO;
    static const unsigned int OK     = IDOK;
    static const unsigned int RETRY  = IDRETRY;
    static const unsigned int YES    = IDYES;
  };

private:
  static std::map<DWORD, TtMessageBox*> hook_table;
  static std::map<DWORD, WNDPROC>       procedure_table;

  static LRESULT CALLBACK Hook( int code, WPARAM w_param, LPARAM l_param );
  static LRESULT CALLBACK Procedure( HWND handle, UINT msg, WPARAM w_param, LPARAM l_param );

public:
  explicit TtMessageBox( const std::string& message = "",
                         const std::string& caption = "",
                         unsigned int       icon    = Icon::NONE );

  const std::string& GetMessage( void ) const;
  void SetMessage( const std::string& message );
  void AppendMessage( const std::string& message );
  TtString::Appender AppendMessage( void );

  const std::string& GetCaption( void ) const;
  void SetCaption( const std::string& caption );

  unsigned int GetIcon( void ) const;
  void SetIcon( unsigned int icon );

  unsigned int GetDefaultButton( void ) const;
  void SetDefaultButton( unsigned int button );

  virtual unsigned int ShowDialog( void );
  virtual unsigned int ShowDialog( TtWindow& parent );

protected:
  virtual unsigned int GetType( void ) const = 0;

private:
  std::string message_;
  std::string caption_;
  unsigned int icon_;
  unsigned int default_button_;

  HHOOK     hook_;
  TtWindow* parent_;
};

// -- TtMessageBoxWithType ----------------------------------------------
template <unsigned int type>
class TtMessageBoxWithType : public TtMessageBox {
public:
  static unsigned int Show( const std::string& message,
                            const std::string& caption = "",
                            unsigned int       icon    = TtMessageBox::Icon::NONE ) {
    return TtMessageBoxWithType<type>( message, caption, icon ).ShowDialog();
  }

  static unsigned int Show( TtWindow& parent,
                            const std::string& message,
                            const std::string& caption = "",
                            unsigned int       icon    = TtMessageBox::Icon::NONE ) {
    return TtMessageBoxWithType<type>( message, caption, icon ).ShowDialog( parent );
  }

public:
  explicit TtMessageBoxWithType<type>( const std::string& message = "",
                                       const std::string& caption = "",
                                       unsigned int       icon    = TtMessageBox::Icon::NONE ) :
  TtMessageBox( message, caption, icon ) {
  }

protected:
  virtual unsigned int GetType( void ) const {
    return type;
  }
};

typedef TtMessageBoxWithType<MB_OK>               TtMessageBoxOk;
typedef TtMessageBoxWithType<MB_OKCANCEL>         TtMessageBoxOkCancel;
typedef TtMessageBoxWithType<MB_RETRYCANCEL>      TtMessageBoxRetryCancel;
typedef TtMessageBoxWithType<MB_YESNO>            TtMessageBoxYesNo;
typedef TtMessageBoxWithType<MB_YESNOCANCEL>      TtMessageBoxYesNoCancel;
typedef TtMessageBoxWithType<MB_ABORTRETRYIGNORE> TtMessageBoxAbortRetryIgnore;

#endif // TT_MESSAGE_BOX_H_
