#ifndef TT_MESSAGE_QUEUE_H_
#define TT_MESSAGE_QUEUE_H_

#include <windows.h>
#include <limits.h>
#include <queue>

#include "ttl_define.h"
#include "tt_exception.h"
#include "tt_critical_section.h"

template <class TYPE>
class TtMessageQueue {
public:
  explicit TtMessageQueue() :
  queue_(),
  lock_(),
  get_semaphoer_() {
    get_semaphoer_ = ::CreateSemaphore( NULL, queue_.size(), INT_MAX, NULL );
    if ( get_semaphoer_ == NULL ) {
      throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::CreateSemaphore ) );
    }
  }

  ~TtMessageQueue() {
    CloseHandle( get_semaphoer_ );
  }

  void PostMessage( TYPE data ) {
    TtCriticalSectionHandler handler( lock_ );
    queue_.push( data );
    if ( ::ReleaseSemaphore( get_semaphoer_, 1, NULL ) == 0 ) {
      throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::ReleaseSemaphore ) );
    }
  }

  bool HasMessage( void ) const {
    return NOT( queue_.empty() );
  }

  TYPE GetMessage( void ) {
    if ( ::WaitForSingleObject( get_semaphoer_, INFINITE ) != WAIT_OBJECT_0 ) {
      throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::WaitForSingleObject ) );
    }
    TtCriticalSectionHandler handler( lock_ );
    TYPE data = queue_.front();
    queue_.pop();
    return data;
  }

  void ClearMessage( void ) {
    TtCriticalSectionHandler handler( lock_ );
    while ( NOT( queue_.empty() ) ) {
      queue_.pop();
    }
  }

private:
  std::queue<TYPE>  queue_;
  TtCriticalSection lock_;
  HANDLE            get_semaphoer_;
};

#endif // TT_MESSAGE_QUEUE_H_
