#ifndef TT_PROCESS_H_
#define TT_PROCESS_H_

#include <string>
#include <windows.h>

#include "tt_enum.h"
#include "tt_window.h"

// -- TtProcess ----------------------------------------------------------
class TtProcess {
public:
  static DWORD GetCurrentProcessId( void );

public:
  // -- Priority ---------------------------------------------------------
  class Priority {
  public:
    enum Value {
      NORMAL       = NORMAL_PRIORITY_CLASS,
      IDLE         = IDLE_PRIORITY_CLASS,
      HIGH         = HIGH_PRIORITY_CLASS,
      REALTIME     = REALTIME_PRIORITY_CLASS,
      BELOW_NORMAL = BELOW_NORMAL_PRIORITY_CLASS,
      ABOVE_NORMAL = ABOVE_NORMAL_PRIORITY_CLASS,
    };
  };

  // -- CreateInfo --------------------------------------------------------
  class CreateInfo {
    friend class TtProcess;
  public:
    explicit CreateInfo( const std::string& filename );

    void SetFileName( const std::string& filename );
    const std::string& GetFileName( void ) const;

    void SetArguments( const std::string& arg );
    const std::string& GetArguments( void ) const;

    void SetCurrentDirectory( const std::string& path );
    const std::string& GetCurrentDirectory( void ) const;

    void SetInheritHandles( bool flag );
    bool GetInheritHandles( void ) const;

    void SetInheritStandardStreams( bool flag );
    bool GetInheritStandardStreams( void ) const;

    void SetCreateNewProcessGroup( bool flag );
    bool GetCreateNewProcessGroup( void ) const;

    void SetPriority( TtEnum<Priority> priority );
    TtEnum<Priority> GetPriority( void ) const;

    void SetShowState( TtEnum<TtWindow::ShowState> state );
    TtEnum<TtWindow::ShowState> GetShowState( void ) const;

    void SetUseSearchPath( bool flag );
    bool GetUseSearchPath( void ) const;

    void SetCreateSuspended( bool flag );
    bool GetCreateSuspended( void ) const;

  private:
    std::string                 filename_;
    std::string                 arguments_;
    std::string                 current_directory_;
    bool                        inherit_handles_;
    bool                        inherit_standard_streams_;
    bool                        create_new_process_group_;
    TtEnum<Priority>            priority_;
    TtEnum<TtWindow::ShowState> show_state_;
    bool                        use_search_path_;
    bool                        create_suspended_;
  };

  // -- TtProcess --------------------------------------------------------
  explicit TtProcess( void );
  ~TtProcess();

  void Create( CreateInfo& info );
  void Start( void );
  void Wait( void );
  void Terminate( UINT exit_code );

  bool HasExited( void ) const;

  DWORD GetExitCode( void ) const;
  DWORD GetProcessId( void ) const;

private:
  HANDLE process_handle_;
  DWORD  process_id_;
  HANDLE thread_handle_;
  DWORD  thread_id_;
};

#endif // TT_PROCESS_H_
