#ifndef TT_PROPERTY_SHEET_H_
#define TT_PROPERTY_SHEET_H_

#include <vector>
#include <map>

#include "tt_form.h"

class TtPropertySheet;

// -- TtPropertySheetPage ------------------------------------------------
class TtPropertySheetPage : public TtForm {
public:
  friend class TtPropertySheet;

  static BOOL CALLBACK WindowProcedureForTTLPropertySheetPage( HWND handle, UINT msg, WPARAM w_param, LPARAM l_param );
  static UINT CALLBACK ProcedureForTTLPropertySheetPage( HWND handle, UINT msg, LPPROPSHEETPAGE page );

protected:
  virtual DWORD  GetStyle( void );
  virtual DWORD  GetExtendedStyle( void );

  virtual bool AtPageCreate( void );
  virtual bool Created( void );
  virtual void AtPageRelease( void );

  virtual LRESULT DefaultWindowProcedure( UINT msg, WPARAM w_param, LPARAM l_param );
  virtual LRESULT WMDestroyReceived( void );
  virtual LRESULT WMNotifyReceived( int id, int code, HWND child );

  virtual void AtSetActive( void );
  virtual bool AtKillActive( void );
  virtual bool AtApply( void );
  virtual void AtReset( void );

public:
  virtual LRESULT WindowProcedure( UINT msg, WPARAM w_param, LPARAM l_param );

  explicit TtPropertySheetPage( TtPropertySheet& sheet, const std::string& title = "" );
  void PresetSizeAtPageCreate( int width, int height );

  virtual ~TtPropertySheetPage();

  const std::string& GetTitle( void ) const;

  virtual void CreatePrimitive( CreateParameter& parameter );

  void TellChanged( void );

private:
  PROPSHEETPAGE    page_;
  std::string      title_;
  void*            template_object_;
};


// -- TtPropertySheet ----------------------------------------------------
class TtPropertySheet : public TtForm {
  friend TtPropertySheetPage::TtPropertySheetPage( TtPropertySheet& sheet, const std::string& title );

private:
  static std::map<DWORD, TtPropertySheet*> TABLE;

public:
  static int CALLBACK ProcedureForTTLPropertySheet( HWND handle, UINT msg, LPARAM l_param );

public:
  explicit TtPropertySheet( bool has_apply_button = true );

  virtual void CreatePrimitive( CreateParameter& parameter );

  void SetTitle( const std::string& title );
  const std::string& GetTitle( void ) const;

  void SetHasApplyButton( bool has_apply_button );
  bool GetHasApplyButton( void ) const;

  bool ShowDialog( TtForm& parent );

private:
  std::vector<TtPropertySheetPage*> pages_;
  bool                              has_apply_button_;
};

#endif // TT_PROPERTY_SHEET_H_
