#ifndef TT_TREEVIEW_H_
#define TT_TREEVIEW_H_

#include "tt_window.h"

// #define TreeView_GetItemRect(hwnd, hitem, prc, code)
// #define TreeView_GetIndent(hwnd)
// #define TreeView_SetIndent(hwnd, indent)
// #define TreeView_GetImageList(hwnd, iImage)
// #define TreeView_SetImageList(hwnd, himl, iImage)
// #define TreeView_Select(hwnd, hitem, code)
// #define TreeView_SelectDropTarget(hwnd, hitem)
// #define TreeView_SelectSetFirstVisible(hwnd, hitem)
// #define TreeView_GetEditControl(hwnd)
// #define TreeView_HitTest(hwnd, lpht)
// #define TreeView_CreateDragImage(hwnd, hitem)
// #define TreeView_EnsureVisible(hwnd, hitem)
// #define TreeView_EndEditLabelNow(hwnd, fCancel)
// #define TreeView_SetToolTips(hwnd,  hwndTT)
// #define TreeView_GetToolTips(hwnd)
// #define TreeView_GetISearchString(hwndTV, lpsz)
// #define TreeView_SetInsertMark(hwnd, hItem, fAfter)
// #define TreeView_SetUnicodeFormat(hwnd, fUnicode)
// #define TreeView_GetUnicodeFormat(hwnd)
// #define TreeView_SetItemHeight(hwnd,  iHeight)
// #define TreeView_GetItemHeight(hwnd)
// #define TreeView_SetBkColor(hwnd, clr)
// #define TreeView_SetTextColor(hwnd, clr)
// #define TreeView_GetBkColor(hwnd)
// #define TreeView_GetTextColor(hwnd)
// #define TreeView_SetScrollTime(hwnd, uTime)
// #define TreeView_GetScrollTime(hwnd)
// #define TreeView_SetInsertMarkColor(hwnd, clr)
// #define TreeView_GetInsertMarkColor(hwnd)
// #define TreeView_SetItemState(hwndTV, hti, data, _mask)
// #define TreeView_SetCheckState(hwndTV, hti, fCheck)
// #define TreeView_GetItemState(hwndTV, hti, mask)
// #define TreeView_GetCheckState(hwndTV, hti)
// #define TreeView_SetLineColor(hwnd, clr)
// #define TreeView_GetLineColor(hwnd)

class TtTreeView;

// -- TtTreeItem ---------------------------------------------------------
class TtTreeItem {
  friend class TtTreeView;

private:
  explicit TtTreeItem( TtTreeView& tree, HTREEITEM handle );

  void GetInfo( TVITEM& info );
  void SetInfo( TVITEM& info );

public:
  HTREEITEM GetHandle( void );

  TtTreeItem GetParent( void );
  TtTreeItem GetFirstChild( void );
  TtTreeItem GetNextBrother( void );
  TtTreeItem GetPrevBrother( void );

private:
  TtTreeItem AddItem( HTREEITEM prev, const std::string& text );
public:
  TtTreeItem AddItemAfter( TtTreeItem& prev, const std::string& text = "" );
  TtTreeItem AddItemFirst( const std::string& text = "" );
  TtTreeItem AddItemLast( const std::string& text = "" );
  TtTreeItem AddItemSorted( const std::string& text = "" );

  void Remove( void );

  void SortChildren( void );
  void SortChildrenRecursive( void );

  std::string GetText( void );
  void SetText( const std::string& str );

  void Expand( void );
  void Collapse( void );
  void ToggleExpand( void );

  void SetEditMode( void );
  void SetSelect( void );

  void SetImageIndex( int index );
  void SetSelectedImageIndex( int index );
  void SetBothImageIndex( int index );

private:
  HTREEITEM   handle_;
  TtTreeView& tree_;
};

// -- TtTreeView ---------------------------------------------------------
class TtTreeView : public BasedOnTtWindow<WS_TABSTOP | TVS_DISABLEDRAGDROP | TVS_HASLINES | TVS_HASBUTTONS | TVS_SHOWSELALWAYS,
                   WS_EX_CLIENTEDGE, TtWindowClassName::TREEVIEW> {
public:
  explicit TtTreeView( void );

  TtTreeItem& GetRoot( void );
  TtTreeItem GetSelected( void );
  TtTreeItem GetDropHilight( void );

  int GetCount( void );
  int GetVisibleCount( void );

  void SetImageList( TtImageList& image_list );
  void SetImageListState( TtImageList& image_list );

private:
  TtTreeItem root_;
};

#endif // TT_TREEVIEW_H_
