#ifndef TT_UTIL_H_
#define TT_UTIL_H_

#include <string>
#include <sstream>

namespace TtUtil {
  bool StringToInteger( const std::string& str, int* ret, unsigned int base );
  bool StringToDouble( const std::string& str, double* ret );

  template <class TYPE>
  std::string ToStringFrom( const TYPE& value ) {
    std::ostringstream os;
    os << value;
    return os.str();
  }

  class NonCopyable {
  protected:
    explicit NonCopyable( void ) {}
    ~NonCopyable() {}
  private:
    explicit NonCopyable( const NonCopyable& );
    const NonCopyable& operator =( const NonCopyable& );
  };

  template <class TYPE>
  TYPE& Bigger( TYPE& r, TYPE& l ) {
    return r > l ? r : l;
  }

  template <class TYPE>
  const TYPE& ConstBigger( const TYPE& r, const TYPE& l ) {
    return r > l ? r : l;
  }

  template <class TYPE>
  TYPE& Smaller( TYPE& r, TYPE& l ) {
    return r > l ? l : r;
  }

  template <class TYPE>
  const TYPE& ConstSmaller( const TYPE& r, const TYPE& l ) {
    return r > l ? l : r;
  }

  std::string GetWindowsSystemErrorMessage( unsigned int error_number );

  std::string BccCompilerDateMacroToNormalString( const std::string& date );
#define TTL_DATE_STRING TtUtil::BccCompilerDateMacroToNormalString( __DATE__ )

  void MessageBoxForDebug( const char* type, const char* obj_name, void* obj, const char* file, int line );
}

#endif // TT_UTIL_H_
