#ifndef TT_WINDOW_H_
#define TT_WINDOW_H_
#define OEMRESOURCE

#include <windows.h>
#include <commctrl.h>

#include <string>
#include <map>
#include <vector>

#include "tt_enum.h"
#include "tt_image_list.h"
#include "tt_font.h"

#define TT_WINDOW_MSG_DEFAULT    0x8000
#define TT_WINDOW_MSG_USER_START (TT_WINDOW_MSG_DEFAULT + 1)

// -- TtWindow -----------------------------------------------------------
class TtWindow {
public:
  static const int MSG_DEFAULT;

public:
  static void InitializeCommonControls( void );
  static LRESULT CALLBACK WindowProcedureForTTL( HWND handle, UINT msg, WPARAM w_param, LPARAM l_param );
  static TtWindow* FindWindow( HWND handle );

protected:
  static std::map<HWND, TtWindow*>& GetTtWindowTable( void );

private:
  static std::map<HWND, TtWindow*> TT_WINDOW_TABLE;

  // ---------------------------------------------------------------------
  // ȉ̃o֐͌pNXŃI[o[Ch邱
protected:
  virtual DWORD  GetStyle( void ) = 0;
  virtual DWORD  GetExtendedStyle( void ) = 0;
  virtual PCTSTR GetClassName( void ) = 0;
  virtual bool Created( void );

public:
  virtual LRESULT WindowProcedure( UINT msg, WPARAM w_param, LPARAM l_param );
  virtual LRESULT WMHorizontalScrollReceived( int code, int position, HWND handle );
  virtual LRESULT WMVerticalScrollReceived( int code, int position, HWND handle );
  virtual LRESULT DefaultWindowProcedure( UINT msg, WPARAM w_param, LPARAM l_param );

  explicit TtWindow( void );
  virtual ~TtWindow();

  // -- CreateParameter --------------------------------------------------
  struct CreateParameter {
  public:
    explicit CreateParameter( void );
    explicit CreateParameter( TtWindow* parent );
    explicit CreateParameter( TtWindow* parent, int id );
    explicit CreateParameter( const CreateParameter& other );

    CreateParameter AlterParent( TtWindow* parent ) const;
    CreateParameter AlterId( int id ) const;
    CreateParameter AlterInstanceHandle( HINSTANCE instance_handle ) const;
    CreateParameter AlterLPParam( LPVOID lp_param ) const;
    CreateParameter AlterIsChild( bool is_child ) const;
    CreateParameter AlterGroupStart( bool group_start ) const;

  public:
    TtWindow* parent_;
    int       id_;
    HINSTANCE instance_handle_;
    LPVOID    lp_param_;
    bool      is_child_;
    bool      group_start_;
  };

  void Create( void );
  void Create( CreateParameter parameter );
  virtual void CreatePrimitive( CreateParameter& parameter );

  int Close( void );

  int SendMessage( UINT msg, WPARAM w_param = 0, LPARAM l_param = 0 );
  void PostMessage( UINT msg, WPARAM w_param = 0, LPARAM l_param = 0 );
  LONG GetWindowLong( int index );
  void SetWindowLong( int index, LONG value );
  void OverrideWindowProcedure( void );

  TtWindow* GetParent( void );
  HWND      GetHandle( void );
  int       GetID( void );
  HINSTANCE GetInstanceHandle( void );

  RECT GetRectangle( void );
  int GetHorizontalPosition( void );
  int GetVerticalPosition( void );
  int GetWidth( void );
  int GetHeight( void );

  // -- ShowState --------------------------------------------------------
  class ShowState {
  public:
    enum Value {
      FORCE_MINIMIZE            = SW_FORCEMINIMIZE,
      HIDE                      = SW_HIDE,
      RESTORE                   = SW_RESTORE,
      MINIMIZE                  = SW_MINIMIZE,
      MAXIMIZE                  = SW_MAXIMIZE,
      SHOW                      = SW_SHOW,
      SHOW_NO_ACTIVE            = SW_SHOWNA,
      SHOW_MINIMIZED            = SW_SHOWMINIMIZED,
      SHOW_MINIMIZED_NO_ACTIVE  = SW_SHOWMINNOACTIVE,
      SHOW_RESTORED             = SW_SHOWNORMAL,
      SHOW_RESTORED_NO_ACTIVATE = SW_SHOWNOACTIVATE,
      SHOW_DEFAULT              = SW_SHOWDEFAULT,
    };
  };

  void Show( void );
  void Hide( void );
  void Minimize( void );
  void Maximize( void );
  void Restore( void );
  void SetShowState( TtEnum<ShowState> state );
  bool GetVisible( void );

  bool GetEnabled( void );
  void SetEnabled( bool flag );

  void SetActive( void );
  void SetFocus( void );

  void SetPosition( int x, int y );
  void SetSize( int width, int height );
  void SetPositionSize( int x, int y, int width, int height );
  void SetRectangle( RECT rect );
  void Move( int x, int y, int width, int height, int repaint );
  void SetCenterRelativeToParent( void );

  void SetBottom( void );
  void SetTop( void );
  void SetUnder( TtWindow& window );
  void SetUnder( HWND window );
  void SetTopMost( bool flag );

  void SetFont( TtFont& font );
  void SetIcon( HICON icon, bool is_big );

  int SetForground( void );

  int GetTextLength( void );
  std::string GetText( void );
  void SetText( const std::string& str );

protected:
  TtWindow* parent_;
  HWND      handle_;
  int       id_;
  HINSTANCE instance_handle_;
  WNDPROC   window_procedure_super_;
};

// -- TtExtraordinarilyWindow --------------------------------------------
class TtExtraordinarilyWindow : public TtWindow {
public:
  static TtExtraordinarilyWindow Desktop;

public:
  explicit TtExtraordinarilyWindow( HWND handle );
  explicit TtExtraordinarilyWindow( HWND handle, TtWindow& parent );

protected:
  virtual DWORD  GetStyle( void );
  virtual DWORD  GetExtendedStyle( void );
  virtual PCTSTR GetClassName( void );
};

// -- TtWindowWithStyle --------------------------------------------------
template <class BASE, DWORD style>
class TtWindowWithStyle : public BASE {
protected:
  virtual DWORD GetStyle( void ) {
    return this->BASE::GetStyle() | style;
  }
};

// -- BasedOnTtWindow --------------------------------------------------
template <DWORD style, DWORD extended_style, PCTSTR class_name>
class BasedOnTtWindow : public TtWindow {
protected:
  virtual DWORD GetStyle( void ) {
    return style;
  }
  virtual DWORD  GetExtendedStyle( void ) {
    return extended_style;
  }
  virtual PCTSTR GetClassName( void ) {
    return class_name;
  }
};

namespace TtWindowClassName {
  extern const char* STATIC;
  extern const char* BUTTON;
  extern const char* EDIT;
  extern const char* LISTBOX;
  extern const char* COMBOBOX;
  extern const char* SCROLLBAR;
  extern const char* TRACKBAR;
  extern const char* TABCONTROL;
  extern const char* PROGRESS;
  extern const char* LISTVIEW;
  extern const char* TREEVIEW;
}

#endif // TT_WINDOW_H_
